//==============================================================================
//
//   SENHA_CR_INFRANEC_FIX
//
//   AUTOR: CLAUDIO LRIOS
//   DATA: 01/03/2014
//   SENHA ELETRONICA USANDO CONTROLE REMOTO INFRAVERMELHO
//   OBJETIVO: ACIONAR FECHADURA DE UM RECINTO, FAZENDO USO DE UM CONTROLE REMO-
//   INFRAVERMELHO COMERCIAL E DIGITANDO UMA SENHA DE 4 DGITOS. 
//
//   MICROCONTROLADOR : PIC 12F675  - SENSOR TSOP 1550 (ou similar)
//
//   PROGRAMAR A SENHA:
//   APERTE MOMENTNEAMENTE O BOTO 'PROG' NA PLACA. ACENDER O LED 'PROG'.
//   APONTE O CONTROLE REMOTO PARA O RECEPTOR INFRA E DIGITE EM SEQUNCIA,
//   4 TECLAS QUAISQUER DO CONTROLE REMOTO (PADRO NEC). AO DIGITAR A QUARTA
//   TECLA, APAGAR O 'LED PROG' E ACIONAR O RELE DA FECHADURA.
//
//   USO: APONTE O CONTROLE REMOTO, J COM AS TECLAS PROGRAMADAS PARA O RECEPTOR
//   E DIGITE AS 4 TECLAS NA MESMA SEQUNCIA QUE FORAM PROGRAMADAS.
//   DEVER ACIONAR A SADA E PISCAR O LED 'ON'. 
//
//   Obs.Somente para protocolo 'NEC' (LG,SEMP)
//   TESTADO COM CONTROLES:
//   DE VCR LG  COD. 6711R1N156A / DE TV LG COD.AKB72915252
//   DE DVD LG COD. 6711R1P089B / DE VCR SEMP COD. CR-X680
//   DE SATLITE ELSYS COD. AS-07552838
//   UNIVERSAL RCA SYSTEMLINK 4 PROGRAMAR P/ 'TV' CODIGO 004
//
//
//==============================================================================
//   Obs. Este dispositivo  apenas didtico, sendo montado apenas em protoboard
//   e sendo sujeito a bugs no identificados.
//
//==============================================================================
//Descomente o abaixo SOMENTE se perdeu o byte de calibrao
 //   #define PERDI_BYTE_CALIBRACAO
//==============================================================================


#include <12F675.h>
#device adc=8
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOCPD, NOPROTECT, NOMCLR, PUT, BROWNOUT
#use fast_io(A)  //  sentido das portas feita pelo programador
#ifdef  PERDI_BYTE_CALIBRACAO  // calibrao do oscilador interno do pic
#ROM  0X3FF = {0X34fc}//mxima freq= 0x34fc ;mdia=0x3480; minima=0x3400
#endif
#ZERO_RAM



//==============================================================================
//   REGISTRADORES
//==============================================================================
#byte gpio   = 0x05
#byte tmr0   = 0x01
#byte tris   = 0x85
#byte t1con   =0x10
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte pir1    =0x0c
#byte tmr1h   =0x0f
#byte tmr1l   =0x0e
#byte intcon  =0x8b
#byte status  =0x03
#byte wpu     =0x95
//==============================================================================
//   DEFINIO DE PINOS E SAIDAS
//==============================================================================
#bit  rele     = 0x5.4 //pino 3
#bit  led_prog = 0x5.1 //pino 6
#bit  s4       = 0x5.0 //pino 7
#bit  _prog    = 0x5.2 //pino 5
#bit  led      = 0x5.5 //pino 2
#bit  _in      = 0x5.3 //pino 4

#bit  cy          = 0x3.0 //carry bit
#bit  on_tmr1     = 0x10.0  //on/off tmr1
#bit  fov1        = 0x0c.0//flag over flow do tmr1
#bit  tmr1if      =  0x0c.0 //flag over flow do tmr1
#bit  tmr0if      = 0x8b.2//flag over tmr0
#bit  pull_prog   =0x95.2//ligar pullup no boto 'prog'


#define in  pin_a3


//#priority RA,timer0
//==============================================================================
//   CONSTANTES GERAIS
//==============================================================================

int const  qbits = 32;  //quantidade de bits a receber
int const  TLIGADO=30; //tempo mnimo para manter acionamento com boto apertado
int const  TINITPROG=150; // tempo mnimo para iniciar 'modo de programao'
int const  comp_senha=4;  // nmero de digitos da senha
int const  tempo1=255;  // tempo para digitao (low)
int const  tempo2= 255; //tempo para digitao (high)


//==============================================================================
//escolha o tempo de acionamento da fechadura

int const tempo_acionamento = 25;  // para 5 segundos

//==============================================================================
// escolha acionamento pulsado ou continuo

#define pulsado

// comente para acionamento sem pulsar
//==============================================================================

//   VARIVEIS GLOBAIS
//==============================================================================

int qb,ax1,cnt1,cnt2,cnt3,cnt4,cnt5,cntp1,T1,T2,dig,command;
int buffer[4];
int buffer1[comp_senha];
short flag_rok;


//==============================================================================
//   SUBROTINAS E FUNES GERAIS
//==============================================================================



//------------------------------------------------------------------------------
//           COMPARAR SENHA RECEBIDA COM SENHA DA EEPROM
//------------------------------------------------------------------------------
short comparar() {
  int a;
  if (command!=read_eeprom(5)){  return (false);}
  for  (a=0;a<comp_senha;++a){
        if((buffer1[a]!= read_eeprom(a))){  return (false);}
  }
   return (true);
  
}
 //==============================================================================
//   INTERRUPO POR MUDANA DE ESTADO - RECEPO DO CONTROLE IR (PADRO 'NEC')
//==============================================================================
#int_RA
void rx_nec_mudanca_gp3() {
 int16 const LIMITE_BYTE_LOW=1200, PULSO_9MS=7000,PULSO_4MS=3800;
 static int16 ref,gt;
 static short ov1,flag_p;


    gpio|=gpio; //ler port para conseguir zera gpif (DS41190C-page 21)
       if(!flag_rok){
           on_tmr1=0; ov1=tmr1if; gt=get_timer1(); set_timer1(0);
           tmr1if=0;  on_tmr1=1;//desliga timer1/pega valor/zera e religa

           cnt1=TLIGADO;
           cnt2=TLIGADO;
           cnt3=TLIGADO;
           cnt4=TLIGADO;
           cnt5=TLIGADO;

           if(!input(in)){         //input='0'
           if((gt>PULSO_9MS)||(ov1)){ qb=qbits;}

             if(flag_p){
              shift_right(&buffer[0],4,(gt>ref));
               if(--qb==0) { qb=qbits;
     if ((buffer[2]+buffer[3]==0xff)){ flag_rok=1;}//extendido apenas
    //   if (!(buffer[0]+buffer[1]+buffer[2]+buffer[3]+0x2)){ flag_rok=1;}//normal
                }
               }
            }
            else{                   //input='1'
                 if((gt>PULSO_4MS)||(ov1)){ qb=qbits; flag_p=0;}
                 else {
                 if((gt<LIMITE_BYTE_LOW)&& (!ov1)&& (!flag_p)){
                    flag_p=1; qb=qbits; ref=gt; ref<<=1; }

                }
            }

         }//if(!flag_rok)
}//void mudanca_gp3()


//==============================================================================

//==============================================================================
//
//                PROGRAMA PRINCIPAL
//
//==============================================================================

void main() {
//==============================================================================
//   INICIALISAO DE PORTAS E REGISTRADORES
//==============================================================================

   setup_adc_ports(no_analogs);  //somente pic12f675
   SETUP_ADC(ADC_OFF); //DESLIGA ADC
   setup_comparator(NC_NC_NC_NC);
   setup_vref(FALSE);
   setup_timer_1 ( T1_INTERNAL | T1_DIV_BY_1 );
   gpio=0;
   ax1=0;
   port_a_pullups(true);
   pull_prog=1;//liga pullup em 'prog'
   tris=0b001100; //tudo sada exceto gp3
   set_timer1(0); //zera tmr1
   tmr1if=0 ; //apaga flag over flow
   on_tmr1=1;//liga timer1
   enable_interrupts(INT_RA3);//habilita interrupo por mudana de estado
   enable_interrupts(global);
   delay_ms(20);
   cntp1=TINITPROG;
   qb=qbits;//carrega quantidade de bits a receber
   flag_rok=0;
   dig=0;
   led=1;
//==============================================================================
//                         LOOP PRINCIPAL
//==============================================================================
   while(true){
   
   if(!_prog){  led_prog=1;}//testa se deve programar nova senha
   
      delay_ms(10);   //atraso temporizao correta do desligamento (modo Pulso)
//==============================================================================
//==============================================================================
//  CANCELAMENTO DA SENHA TRANSMITIDA APS TEMPO EXCEDIDO
//==============================================================================
    if(!(--t1)){ if(--t2) {
      int a;
     for (a=0; a<comp_senha;++a){ buffer1[a]=0xff; }
     dig=0;
     }
   }

 //=============================================================================
//   ACIONAR SADAS CONFORME PROGRAMAO
//==============================================================================


 if((flag_rok)){


    t1=tempo1; t2=tempo2; //recarga do tempo de apagamento do buffer
    buffer1[dig++]=buffer[2];// pega tecla digitada para futura comparao
    command=buffer[0];
    led=0;
    delay_ms(100);//pisca para indicar recepo correta do digito
    led=1;
    flag_rok=0; //libera nova recepo de tecla
  }

   if(dig==comp_senha){  //se j recebeu toda a senha vai compara-la com eeprom
    int a;
       if(led_prog) {   //se for modo de programa de senha
       disable_interrupts(global);
         for(a=0;a<4;++a) {
         write_eeprom(a,buffer1[a]);
         }
         write_eeprom(5, command); //escreve o 'command'
         enable_interrupts(global);
       led_prog=0;
       }
    if (comparar()){
   
     for (a=0; a<tempo_acionamento; ++a) {
      rele=1;
      led=0;
      delay_ms(100);
      #ifdef pulsado
      rele=0;
      #endif
      led=1;
      delay_ms(100);
     }
    }//se iguais, libera saida
    
      for (a=0; a<comp_senha;++a){ buffer1[a]=0xff; } //diferentes,apaga
    dig=0; rele=0; led=0;  command=0xff; //buffer

   }

 }//void while(true)
}//void main



